==> kahakai-0.6.1.tar.bz2: kahakai-0.6.1/src/DrawOp.cc <==
/**
 * @file   DrawOp.cc
 * @date   Sat Oct  4 23:09:30 2003
 * 
 * @brief Implementation of DrawOp classes which operate on ImImage such as
 *        drawing lines, rectangles, gradients, and interlacing.
 * 
 */

#include "DrawOp.hh"
#include "ExpressionEvaluator.hh"
#include "Logger.hh"
#include "Texture.hh"

using namespace std;

const int tileWidth  = 64;
const int tileHeight = 24;

DrawOp::DrawOp(): evaluator(variables) {};
DrawOp::~DrawOp() {};

int DrawOp::eval(const ImImage& img, string expr) {
    variables["width"]  = img.get_width();
    variables["height"] = img.get_height();

    try {
        return (int)evaluator.evaluate(expr);
    } catch (parse_exception e) {
        logger << warn << e.what() << endl;        

        return 0;
    }
}

void DrawOpRect::draw(ImImage& img) {
    if (dynamic) {
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);
    }

    img.draw_rectangle(color, x, y, width, height);
}

void DrawOpFillRect::draw(ImImage& img) {
    if (dynamic) {
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);
    }

    img.fill_rectangle(color, x, y, width, height);
}

void DrawOpLine::draw(ImImage& img) {
    if (dynamic) {
        x  = eval(img, expr_x);
        y  = eval(img, expr_y);
        x2 = eval(img, expr_x2);
        y2 = eval(img, expr_y2);
    }

    img.draw_line(color, x, y, x2, y2);
}

DrawOpImage::DrawOpImage(std::string filename_, int _x, int _y, 
                         unsigned char _alpha, ImageDrawType _drawType,
                         unsigned int borderLeft_, 
                         unsigned int borderTop_, 
                         unsigned int borderRight_, 
                         unsigned int borderBottom_):
    filename(filename_), x(_x), y(_y), alpha(_alpha), drawType(_drawType),
    borderLeft(borderLeft_), borderTop(borderTop_), borderRight(borderRight_),
    borderBottom(borderBottom_) { };

void DrawOpImage::setBorders(ImImage& image) {
    if (borderLeft || borderRight || borderTop || borderBottom) {
        unsigned int w = image.get_width(), h = image.get_height();
        
        borderLeft  = min(w, borderLeft);
        borderRight = min(w, borderRight);
        if (borderLeft + borderRight > w) {
            borderRight = w - borderLeft - 1;
        }
        
        borderTop    = min(h, borderTop);
        borderBottom = min(h, borderBottom);
        if (borderTop + borderBottom > h) {
            borderBottom = h - borderTop - 1;
        }
        image.set_border(borderLeft, borderTop, borderRight, borderBottom);
    } else {
        image.set_border(image.get_width() / 2, 
                         image.get_height() / 2,
                         image.get_width() / 2 - 1,
                         image.get_height() / 2 - 1);            
    }    
}

void DrawOpImage::draw(ImImage& canvas) {   
    ImImage image(filename);
    image.apply_fixed_alpha(alpha);

    if (drawType == DrawTiled) {
        unsigned int width   = image.get_width(), height = image.get_height();
        unsigned int columns = canvas.get_width() / width + 1;
        unsigned int rows    = canvas.get_height() / height + 1;
        
        for (unsigned int y=0;y<rows;y++)
            for (unsigned int x=0;x<columns;x++)
                canvas.blend(image, true, x * width, y * height);
    } else {
        if (drawType == DrawStretched) setBorders(image);

        canvas.blend_scaled(image, true, x, y, 
                            canvas.get_width(), canvas.get_height());
    }
}

void DrawOpBevel::draw(ImImage& img) {
    img.draw_bevel(color, width, bevelType);
}

DrawOpInterlace::DrawOpInterlace(unsigned char _whiteAlpha, unsigned char _blackAlpha, const ImColor& baseColor):
    whiteAlpha(_whiteAlpha), blackAlpha(_blackAlpha) {
    
    // Create a tileWidth x tileheight interlace image
    ImColor light = baseColor.get_lolite();
    light.alpha = 128; //whiteAlpha;
    ImColor dark = baseColor.get_hilite();
    dark.alpha = 64; //blackAlpha;

    tile = new ImImage(tileWidth, tileHeight);
    tile->clear(ImColor(0, 0, 0, 0));
    tile->set_has_alpha(true);

    for (int y=1;y<tileHeight;y+=2) {
      tile->draw_line(dark,  0, y-1,   tileWidth, y-1);
      //      tile->draw_line(light, 0, y,     tileWidth, y);
    }
}

DrawOpInterlace::~DrawOpInterlace() {
    delete tile;
}


void DrawOpInterlace::draw(ImImage& img) {
    // We round up because we want tile to cover the remainder.
    unsigned int columns = img.get_width() / tileWidth + 1;
    unsigned int rows    = img.get_height() / tileHeight + 1;

    for (unsigned int y=0;y<rows;y++)
        for (unsigned int x=0;x<columns;x++)
            img.blend(*tile, true, x * tileWidth, y * tileHeight);
}

void DrawOpGradient::draw(ImImage& img) {
    if (dynamic) {        
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);        
    } 

    img.fill_color_range_rectangle(colorRange, x, y, width, height,
                                   angle);
}

void DrawOpTexture::draw(ImImage& img) {
    if (!texture) return;

    if (dynamic) {
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);
    }

    shared_ptr<ImImage> textureImage = texture->render(width, height);
    img.blend(*textureImage, true, x, y);
}

==> kahakai-0.6.2.tar.bz2: kahakai-0.6.2/src/DrawOp.cc <==
/**
 * @file   DrawOp.cc
 * @date   Sat Oct  4 23:09:30 2003
 * 
 * @brief Implementation of DrawOp classes which operate on ImImage such as
 *        drawing lines, rectangles, gradients, and interlacing.
 * 
 */

#include "DrawOp.hh"
#include "ExpressionEvaluator.hh"
#include "Logger.hh"
#include "Texture.hh"

using namespace std;

const int tileWidth  = 64;
const int tileHeight = 24;

DrawOp::DrawOp(): evaluator(variables) {};
DrawOp::~DrawOp() {};

int DrawOp::eval(const ImImage& img, string expr) {
    variables["width"]  = img.get_width();
    variables["height"] = img.get_height();

    try {
        return (int)evaluator.evaluate(expr);
    } catch (parse_exception e) {
        logger << warn << e.what() << endl;        

        return 0;
    }
}

void DrawOpRect::draw(ImImage& img) {
    if(! img.valid())
        return;
    
    if (dynamic) {
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);
    }

    img.draw_rectangle(color, x, y, width, height);
}

void DrawOpFillRect::draw(ImImage& img) {
    if(! img.valid())
        return;
    
    if (dynamic) {
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);
    }

    img.fill_rectangle(color, x, y, width, height);
}

void DrawOpLine::draw(ImImage& img) {
    if(! img.valid())
        return;
    
    if (dynamic) {
        x  = eval(img, expr_x);
        y  = eval(img, expr_y);
        x2 = eval(img, expr_x2);
        y2 = eval(img, expr_y2);
    }

    img.draw_line(color, x, y, x2, y2);
}

DrawOpImage::DrawOpImage(std::string filename_, int _x, int _y, 
                         unsigned char _alpha, ImageDrawType _drawType,
                         unsigned int borderLeft_, 
                         unsigned int borderTop_, 
                         unsigned int borderRight_, 
                         unsigned int borderBottom_):
    filename(filename_), x(_x), y(_y), alpha(_alpha), drawType(_drawType),
    borderLeft(borderLeft_), borderTop(borderTop_), borderRight(borderRight_),
    borderBottom(borderBottom_) { };

void DrawOpImage::setBorders(ImImage& image) {
    if(! image.valid())
        return;
    
    if (borderLeft || borderRight || borderTop || borderBottom) {
        unsigned int w = image.get_width(), h = image.get_height();
        
        borderLeft  = min(w, borderLeft);
        borderRight = min(w, borderRight);
        if (borderLeft + borderRight > w) {
            borderRight = w - borderLeft - 1;
        }
        
        borderTop    = min(h, borderTop);
        borderBottom = min(h, borderBottom);
        if (borderTop + borderBottom > h) {
            borderBottom = h - borderTop - 1;
        }
        image.set_border(borderLeft, borderTop, borderRight, borderBottom);
    } else {
        image.set_border(0, 0, 0, 0);
    }    
}

void DrawOpImage::draw(ImImage& canvas) {
    if(! canvas.valid())
        return;
    
    ImImage image(filename);
    if(! image.valid())
        return;
        
    image.apply_fixed_alpha(alpha);

    if (drawType == DrawTiled) {
        unsigned int width   = image.get_width(), height = image.get_height();
        unsigned int columns = canvas.get_width() / width + 1;
        unsigned int rows    = canvas.get_height() / height + 1;
        
        for (unsigned int y=0;y<rows;y++)
            for (unsigned int x=0;x<columns;x++)
                canvas.blend(image, true, x * width, y * height);
    } else {
        if (drawType == DrawStretched) setBorders(image);

        canvas.blend_scaled(image, true, x, y, 
                            canvas.get_width() - x, canvas.get_height() - y);
    }
}

void DrawOpBevel::draw(ImImage& img) {
    if(! img.valid())
        return;
    img.draw_bevel(color, width, bevelType);
}

DrawOpInterlace::DrawOpInterlace(unsigned char _whiteAlpha, unsigned char _blackAlpha, const ImColor& baseColor):
    whiteAlpha(_whiteAlpha), blackAlpha(_blackAlpha) {
    
    // Create a tileWidth x tileheight interlace image
    ImColor light = baseColor.get_lolite();
    light.alpha = 128; //whiteAlpha;
    ImColor dark = baseColor.get_hilite();
    dark.alpha = 64; //blackAlpha;

    tile = new ImImage(tileWidth, tileHeight);
    tile->clear(ImColor(0, 0, 0, 0));
    tile->set_has_alpha(true);

    for (int y=1;y<tileHeight;y+=2) {
      tile->draw_line(dark,  0, y-1,   tileWidth, y-1);
      //      tile->draw_line(light, 0, y,     tileWidth, y);
    }
}

DrawOpInterlace::~DrawOpInterlace() {
    delete tile;
}


void DrawOpInterlace::draw(ImImage& img) {
    if(! img.valid())
        return;
    
    // We round up because we want tile to cover the remainder.
    unsigned int columns = img.get_width() / tileWidth + 1;
    unsigned int rows    = img.get_height() / tileHeight + 1;

    for (unsigned int y=0;y<rows;y++)
        for (unsigned int x=0;x<columns;x++)
            img.blend(*tile, true, x * tileWidth, y * tileHeight);
}

void DrawOpGradient::draw(ImImage& img) {
    if(! img.valid())
        return;
    
    if (dynamic) {        
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);        
    } 

    img.fill_color_range_rectangle(colorRange, x, y, width, height,
                                   angle);
}

void DrawOpTexture::draw(ImImage& img) {
    if (! texture || ! img.valid())
        return;

    if (dynamic) {
        x = eval(img, expr_x);
        y = eval(img, expr_y);
        width  = eval(img, expr_width);
        height = eval(img, expr_height);
    }

    shared_ptr<ImImage> textureImage = texture->render(width, height);
    img.blend(*textureImage, true, x, y);
}
